@@ -42,9 +42,7 @@ public:
     bool Cutlines;
     bool Legend;
     wxString title;
-    wxString cs_proj;
     wxString datestamp;
-    time_t datestamp_numeric;
     double Scale;
     double rot, tilt;
     enum {PLAN, ELEV, TILT, EXTELEV} view;
