@@ -1,6 +1,6 @@
 /* cavern.h
  * SURVEX Cave surveying software - header file
- * Copyright (C) 1991-2003,2005,2006,2010,2013,2014,2015,2016,2019,2021 Olly Betts
+ * Copyright (C) 1991-2022 Olly Betts
  * Copyright (C) 2004 Simeon Warner
  *
  * This program is free software; you can redistribute it and/or modify
@@ -341,6 +341,12 @@ typedef struct Settings {
     * Only meaningful if date1 != -1.
     */
    real declination;
+   double min_declination, max_declination;
+   int min_declination_days, max_declination_days;
+   const char* dec_filename;
+   int dec_line;
+   /* Copy of the text of the `*declination auto ...` line (malloced). */
+   char* dec_context;
    /* Grid convergence in radians. */
    real convergence;
    meta_data * meta;
@@ -363,8 +369,8 @@ extern long cLegs, cStns, cComponents;
 extern FILE *fhErrStat;
 extern img *pimg;
 extern real totadj, total, totplan, totvert;
-extern real min[3], max[3];
-extern prefix *pfxHi[3], *pfxLo[3];
+extern real min[6], max[6];
+extern prefix *pfxHi[6], *pfxLo[6];
 extern bool fQuiet; /* just show brief summary + errors */
 extern bool fMute; /* just show errors */
 extern bool fSuppress; /* only output 3d file */
