@@ -1,6 +1,6 @@
 /* commands.c
  * Code for directives
- * Copyright (C) 1991-2003,2004,2005,2006,2010,2011,2012,2013,2014,2015,2016,2019 Olly Betts
+ * Copyright (C) 1991-2022 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -715,34 +715,84 @@ invalidate_pj_cached(void)
     }
 }
 
-void pop_settings()
+void
+report_declination(settings *p)
 {
-   settings * p = pcs;
-   pcs = pcs->next;
-   SVX_ASSERT(pcs);
+    if (p->min_declination <= p->max_declination) {
+	int y, m, d;
+	char range[128];
+	const char* deg_sign = msg(/*°*/344);
+	ymd_from_days_since_1900(p->min_declination_days, &y, &m, &d);
+	snprintf(range, sizeof(range),
+		 "%.1f%s @ %04d-%02d-%02d",
+		 deg(p->min_declination), deg_sign, y, m, d);
+	if (p->min_declination_days != p->max_declination_days) {
+	    size_t len = strlen(range);
+	    ymd_from_days_since_1900(p->max_declination_days, &y, &m, &d);
+	    snprintf(range + len, sizeof(range) - len,
+		     " / %.1f%s @ %04d-%02d-%02d",
+		     deg(p->max_declination), deg_sign, y, m, d);
+	}
+	/* TRANSLATORS: This message gives information about the range of
+	 * declination values and the grid convergence value calculated for
+	 * each "*declination auto ..." command.
+	 *
+	 * The first %s will be replaced by the declination range (or single
+	 * value), and %.1f%s by the grid convergence angle.
+	 */
+	compile_diagnostic_at(DIAG_INFO|DIAG_COL, p->dec_filename, p->dec_line,
+			      /*Declination: %s, grid convergence: %.1f%s*/484,
+			      range,
+			      deg(p->convergence), deg_sign);
+	PUTC(' ', STDERR);
+	fputs(p->dec_context, STDERR);
+	fputnl(STDERR);
+	free(p->dec_context);
+	p->dec_context = NULL;
+    }
+}
 
-   if (p->proj_str != pcs->proj_str) {
-       if (!p->proj_str || !pcs->proj_str ||
-	   strcmp(p->proj_str, pcs->proj_str) != 0) {
-	   invalidate_pj_cached();
-       }
-       /* free proj_str if not used by parent */
-       osfree(p->proj_str);
-   }
+void
+pop_settings(void)
+{
+    settings * p = pcs;
+    pcs = pcs->next;
 
-   /* don't free default ordering or ordering used by parent */
-   if (p->ordering != default_order && p->ordering != pcs->ordering)
-      osfree((reading*)p->ordering);
+    SVX_ASSERT(pcs);
 
-   /* free Translate if not used by parent */
-   if (!pcs || p->Translate != pcs->Translate)
-      osfree(p->Translate - 1);
+    if (pcs->dec_lat != p->dec_lat ||
+	pcs->dec_lon != p->dec_lon ||
+	pcs->dec_alt != p->dec_alt) {
+	report_declination(p);
+    } else {
+	pcs->min_declination_days = p->min_declination_days;
+	pcs->max_declination_days = p->max_declination_days;
+	pcs->min_declination = p->min_declination;
+	pcs->max_declination = p->max_declination;
+    }
 
-   /* free meta if not used by parent, or in this block */
-   if (p->meta && p->meta != pcs->meta && p->meta->ref_count == 0)
-       osfree(p->meta);
+    if (p->proj_str != pcs->proj_str) {
+	if (!p->proj_str || !pcs->proj_str ||
+	    strcmp(p->proj_str, pcs->proj_str) != 0) {
+	    invalidate_pj_cached();
+	}
+	/* free proj_str if not used by parent */
+	osfree(p->proj_str);
+    }
 
-   osfree(p);
+    /* don't free default ordering or ordering used by parent */
+    if (p->ordering != default_order && p->ordering != pcs->ordering)
+	osfree((reading*)p->ordering);
+
+    /* free Translate if not used by parent */
+    if (p->Translate != pcs->Translate)
+	osfree(p->Translate - 1);
+
+    /* free meta if not used by parent, or in this block */
+    if (p->meta && p->meta != pcs->meta && p->meta->ref_count == 0)
+	osfree(p->meta);
+
+    osfree(p);
 }
 
 static void
@@ -854,12 +904,9 @@ cmd_fix(void)
 	 return;
       }
 
-      /* TRANSLATORS: " *fix a " gives this message: */
-      compile_diagnostic(DIAG_WARN|DIAG_COL, /*FIX command with no coordinates - fixing at (0,0,0)*/54);
-
       if (name_omit_already) {
-	 /* TRANSLATORS: Emitted after second and subsequent "FIX command with
-	  * no coordinates - fixing at (0,0,0)" warnings.
+	 /* TRANSLATORS: Emitted after second and subsequent "FIX" command
+	  * with no coordinates.
 	  */
 	 compile_diagnostic_at(DIAG_ERR|DIAG_COL,
 			       name_omit_already_filename,
@@ -867,6 +914,9 @@ cmd_fix(void)
 			       /*Already had FIX command with no coordinates for station “%s”*/441,
 			       sprint_prefix(name_omit_already));
       } else {
+	 /* TRANSLATORS: " *fix a " gives this message: */
+	 compile_diagnostic(DIAG_INFO|DIAG_COL, /*FIX command with no coordinates - fixing at (0,0,0)*/54);
+
 	 name_omit_already = fix_name;
 	 name_omit_already_filename = file.filename;
 	 name_omit_already_line = file.line;
@@ -1763,16 +1813,32 @@ cmd_declination(void)
 	   x = y = z = 0;
 	}
 	proj_destroy(transform);
+
+	report_declination(pcs);
+
 	double lon = rad(x);
 	double lat = rad(y);
 	pcs->z[Q_DECLINATION] = HUGE_REAL;
 	pcs->dec_lat = lat;
 	pcs->dec_lon = lon;
 	pcs->dec_alt = z;
+	pcs->dec_filename = file.filename;
+	pcs->dec_line = file.line;
+	pcs->dec_context = grab_line();
 	/* Invalidate cached declination. */
 	pcs->declination = HUGE_REAL;
 	{
-	    PJ *pj = proj_create(PJ_DEFAULT_CTX, proj_str_out);
+	    // PJ_DEFAULT_CTX is really just NULL, but PROJ < 8.1.0
+	    // dereferences the context without a NULL check inside
+	    // proj_create_ellipsoidal_2D_cs() so create a context
+	    // temporarily to avoid a segmentation fault.
+	    PJ_CONTEXT * ctx = PJ_DEFAULT_CTX;
+#if PROJ_VERSION_MAJOR < 8 || \
+    (PROJ_VERSION_MAJOR == 8 && PROJ_VERSION_MINOR < 1)
+	    ctx = proj_context_create();
+#endif
+
+	    PJ *pj = proj_create(ctx, proj_str_out);
 	    PJ_COORD lp;
 	    lp.lp.lam = lon;
 	    lp.lp.phi = lat;
@@ -1790,10 +1856,10 @@ cmd_declination(void)
 		     * to be consistent with the expectations of the lp input parameter.
 		     */
 
-		    PJ * geodetic_crs = proj_get_source_crs(PJ_DEFAULT_CTX, pj);
+		    PJ * geodetic_crs = proj_get_source_crs(ctx, pj);
 		    if (!geodetic_crs)
 			break;
-		    PJ * datum = proj_crs_get_datum(PJ_DEFAULT_CTX, geodetic_crs);
+		    PJ * datum = proj_crs_get_datum(ctx, geodetic_crs);
 #if PROJ_VERSION_MAJOR == 8 || \
     (PROJ_VERSION_MAJOR == 7 && PROJ_VERSION_MINOR >= 2)
 		    /* PROJ 7.2.0 upgraded to EPSG 10.x which added the concept
@@ -1805,23 +1871,22 @@ cmd_declination(void)
 		     * them.
 		     */
 		    if (!datum) {
-			datum = proj_crs_get_datum_ensemble(PJ_DEFAULT_CTX, geodetic_crs);
+			datum = proj_crs_get_datum_ensemble(ctx, geodetic_crs);
 		    }
 #endif
 		    PJ * cs = proj_create_ellipsoidal_2D_cs(
-			PJ_DEFAULT_CTX, PJ_ELLPS2D_LONGITUDE_LATITUDE, "Radian", 1.0);
+			ctx, PJ_ELLPS2D_LONGITUDE_LATITUDE, "Radian", 1.0);
 		    PJ * temp = proj_create_geographic_crs_from_datum(
-			PJ_DEFAULT_CTX, "unnamed crs", datum, cs);
+			ctx, "unnamed crs", datum, cs);
 		    proj_destroy(datum);
 		    proj_destroy(cs);
 		    proj_destroy(geodetic_crs);
-		    PJ * newOp = proj_create_crs_to_crs_from_pj(PJ_DEFAULT_CTX, temp, pj, NULL, NULL);
+		    PJ * newOp = proj_create_crs_to_crs_from_pj(ctx, temp, pj, NULL, NULL);
 		    proj_destroy(temp);
-		    if (!newOp) {
-			break;
+		    if (newOp) {
+			proj_destroy(pj);
+			pj = newOp;
 		    }
-		    proj_destroy(pj);
-		    pj = newOp;
 		    break;
 		}
 		default:
@@ -1831,6 +1896,10 @@ cmd_declination(void)
 	    PJ_FACTORS factors = proj_factors(pj, lp);
 	    pcs->convergence = factors.meridian_convergence;
 	    proj_destroy(pj);
+#if PROJ_VERSION_MAJOR < 8 || \
+    (PROJ_VERSION_MAJOR == 8 && PROJ_VERSION_MINOR < 1)
+	    proj_context_destroy(ctx);
+#endif
 	}
     } else {
 	/* *declination D UNITS */
@@ -2147,7 +2216,7 @@ cmd_cs(void)
    }
    if (cs_sub == INT_MIN || isalnum(ch)) {
       set_pos(&fp);
-      compile_diagnostic(DIAG_ERR|DIAG_TOKEN, /*Unknown coordinate system*/434);
+      compile_diagnostic(DIAG_ERR|DIAG_STRING, /*Unknown coordinate system*/434);
       skipline();
       return;
    }
@@ -2223,7 +2292,7 @@ cmd_cs(void)
    if (!proj_str) {
       /* printf("CS %d:%d\n", (int)cs, cs_sub); */
       set_pos(&fp);
-      compile_diagnostic(DIAG_ERR|DIAG_TOKEN, /*Unknown coordinate system*/434);
+      compile_diagnostic(DIAG_ERR|DIAG_STRING, /*Unknown coordinate system*/434);
       skipline();
       return;
    }
@@ -2231,7 +2300,7 @@ cmd_cs(void)
    if (output) {
       if (ok_for_output == NO) {
 	 set_pos(&fp);
-	 compile_diagnostic(DIAG_ERR|DIAG_TOKEN, /*Coordinate system unsuitable for output*/435);
+	 compile_diagnostic(DIAG_ERR|DIAG_STRING, /*Coordinate system unsuitable for output*/435);
 	 skipline();
 	 return;
       }
@@ -2251,7 +2320,7 @@ cmd_cs(void)
 	  PJ* pj = proj_create(PJ_DEFAULT_CTX, proj_str);
 	  if (!pj) {
 	      set_pos(&fp);
-	      compile_diagnostic(DIAG_ERR|DIAG_TOKEN, /*Invalid coordinate system: %s*/443,
+	      compile_diagnostic(DIAG_ERR|DIAG_STRING, /*Invalid coordinate system: %s*/443,
 				 proj_errno_string(proj_context_errno(PJ_DEFAULT_CTX)));
 	      skipline();
 	      osfree(proj_str);
@@ -2261,7 +2330,7 @@ cmd_cs(void)
 	  if (type == PJ_TYPE_GEOGRAPHIC_2D_CRS ||
 	      type == PJ_TYPE_GEOGRAPHIC_3D_CRS) {
 	      set_pos(&fp);
-	      compile_diagnostic(DIAG_ERR|DIAG_TOKEN, /*Coordinate system unsuitable for output*/435);
+	      compile_diagnostic(DIAG_ERR|DIAG_STRING, /*Coordinate system unsuitable for output*/435);
 	      skipline();
 	      osfree(proj_str);
 	      return;
@@ -2290,7 +2359,7 @@ cmd_cs(void)
 	 PJ* pj = proj_create(PJ_DEFAULT_CTX, proj_str);
 	 if (!pj) {
 	    set_pos(&fp);
-	    compile_diagnostic(DIAG_ERR|DIAG_TOKEN, /*Invalid coordinate system: %s*/443,
+	    compile_diagnostic(DIAG_ERR|DIAG_STRING, /*Invalid coordinate system: %s*/443,
 			       proj_errno_string(proj_context_errno(PJ_DEFAULT_CTX)));
 	    skipline();
 	    return;
