@@ -2,7 +2,7 @@
  * Export from Aven as HPGL.
  */
 
-/* Copyright (C) 2005,2014,2015 Olly Betts
+/* Copyright (C) 2005,2014,2015,2016 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@ class HPGL : public ExportFilter {
     void header(const char *, const char *, time_t,
 		double, double, double,
 		double, double, double);
-    void line(const img_point *, const img_point *, bool, bool);
+    void line(const img_point *, const img_point *, unsigned, bool);
     void label(const img_point *, const char *, bool, int);
     void cross(const img_point *, bool);
     void footer();
