@@ -86,9 +86,11 @@ class traverse : public vector<PointInfo> {
     int n_legs = 0;
     // Bitmask of img_FLAG_SURFACE, img_FLAG_SPLAY and img_FLAG_DUPLICATE.
     int flags = 0;
+    // An img_STYLE_* value.
+    int style = 0;
     double length = 0.0;
     enum { ERROR_3D = 0, ERROR_H = 1, ERROR_V = 2 };
-    double errors[3] = {0, 0, 0};
+    double errors[3] = {-1, -1, -1};
     wxString name;
 
     explicit
@@ -167,11 +169,11 @@ class Model {
 
     void do_prepare_tubes() const;
 
+    void CentreDataset(const Vector3& vmin);
+
   public:
     int Load(const wxString& file, const wxString& prefix);
 
-    void CentreDataset(const Vector3& vmin);
-
     const Vector3& GetExtent() const { return m_Ext; }
 
     const wxString& GetSurveyTitle() const { return m_Title; }
