@@ -1,7 +1,7 @@
 /* pos.cc
- * Export from Aven as Survex .pos.
+ * Export from Aven as Survex .pos or .csv.
  */
-/* Copyright (C) 2001,2002,2011,2013,2014,2015 Olly Betts
+/* Copyright (C) 2001,2002,2011,2013,2014,2015,2018 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,11 +37,40 @@
 
 using namespace std;
 
+static void
+csv_quote(const char* s, FILE* fh)
+{
+    size_t i = 0;
+    while (true) {
+	switch (s[i]) {
+	    case '\0':
+		fputs(s, fh);
+		return;
+	    case ',':
+	    case '"':
+	    case '\r':
+	    case '\n':
+		break;
+	}
+	++i;
+    }
+    PUTC('"', fh);
+    fwrite(s, i, 1, fh);
+    while (s[i]) {
+	// Double up any " in the string to escape them.
+	if (s[i] == '"')
+	    PUTC(s[i], fh);
+	PUTC(s[i], fh);
+	++i;
+    }
+    PUTC('"', fh);
+}
+
 POS::~POS()
 {
     vector<pos_label*>::const_iterator i;
     for (i = todo.begin(); i != todo.end(); ++i) {
-	delete [] *i;
+	free(*i);
     }
     todo.clear();
 }
@@ -56,16 +85,31 @@ POS::passes() const
 void POS::header(const char *, const char *, time_t,
 		 double, double, double, double, double, double)
 {
-    /* TRANSLATORS: Heading line for .pos file.  Please try to ensure the “,”s
-     * (or at least the columns) are in the same place */
-    fputsnl(msg(/*( Easting, Northing, Altitude )*/195), fh);
+    if (csv) {
+	bool comma = false;
+	for (int msgno : { /*Easting*/378,
+			   /*Northing*/379,
+			   /*Altitude*/335,
+			   /*Station Name*/100 }) {
+	    if (comma) PUTC(',', fh);
+	    csv_quote(msg(msgno), fh);
+	    comma = true;
+	}
+	PUTC('\n', fh);
+    } else {
+	/* TRANSLATORS: Heading line for .pos file.  Please try to ensure the
+	 * “,”s (or at least the columns) are in the same place */
+	fputsnl(msg(/*( Easting, Northing, Altitude )*/195), fh);
+    }
 }
 
 void
 POS::label(const img_point *p, const char *s, bool /*fSurface*/, int /*type*/)
 {
     size_t len = strlen(s);
-    pos_label * l = (pos_label*)new char[offsetof(pos_label, name) + len + 1];
+    pos_label * l = (pos_label*)malloc(offsetof(pos_label, name) + len + 1);
+    if (l == NULL)
+	throw std::bad_alloc();
     l->x = p->x;
     l->y = p->y;
     l->z = p->z;
@@ -90,7 +134,13 @@ POS::footer()
     sort(todo.begin(), todo.end(), pos_label_ptr_cmp(separator));
     vector<pos_label*>::const_iterator i;
     for (i = todo.begin(); i != todo.end(); ++i) {
-	fprintf(fh, "(%8.2f, %8.2f, %8.2f ) %s\n",
-		(*i)->x, (*i)->y, (*i)->z, (*i)->name);
+	if (csv) {
+	    fprintf(fh, "%.2f,%.2f,%.2f,", (*i)->x, (*i)->y, (*i)->z);
+	    csv_quote((*i)->name, fh);
+	    PUTC('\n', fh);
+	} else {
+	    fprintf(fh, "(%8.2f, %8.2f, %8.2f ) %s\n",
+		    (*i)->x, (*i)->y, (*i)->z, (*i)->name);
+	}
     }
 }
