@@ -2,7 +2,7 @@
  * Convert a processed survey data file to another format.
  */
 
-/* Copyright (C) 1994-2004,2008,2010,2011,2013,2014,2018,2020 Olly Betts
+/* Copyright (C) 1994-2004,2008,2010,2011,2013,2014,2018,2020,2022 Olly Betts
  * Copyright (C) 2004 John Pybus (SVG Output code)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -102,6 +102,7 @@ main(int argc, char **argv)
 	{"grid", optional_argument, 0, 'g'},
 	{"text-height", required_argument, 0, 't'},
 	{"marker-size", required_argument, 0, 'm'},
+	{"3d", no_argument, 0, OPT_FMT_BASE + FMT_3D},
 	{"csv", no_argument, 0, OPT_FMT_BASE + FMT_CSV},
 	{"dxf", no_argument, 0, OPT_FMT_BASE + FMT_DXF},
 	{"eps", no_argument, 0, OPT_FMT_BASE + FMT_EPS},
@@ -155,21 +156,22 @@ main(int argc, char **argv)
 	{HLP_ENCODELONG(21),  /*generate grid (default %sm)*/148, STRING(DEFAULT_GRID_SPACING)},
 	{HLP_ENCODELONG(22),  /*station labels text height (default %s)*/149, STRING(DEFAULT_TEXT_HEIGHT)},
 	{HLP_ENCODELONG(23),  /*station marker size (default %s)*/152, STRING(DEFAULT_MARKER_SIZE)},
-	{HLP_ENCODELONG(24),  /*produce CSV output*/102, 0},
-	{HLP_ENCODELONG(25),  /*produce DXF output*/156, 0},
-	{HLP_ENCODELONG(26),  /*produce EPS output*/454, 0},
-	{HLP_ENCODELONG(27),  /*produce GPX output*/455, 0},
-	{HLP_ENCODELONG(28),  /*produce HPGL output*/456, 0},
-	{HLP_ENCODELONG(29),  /*produce JSON output*/457, 0},
-	{HLP_ENCODELONG(30),  /*produce KML output*/458, 0},
+	{HLP_ENCODELONG(24),  /*produce Survex 3d output*/487, 0},
+	{HLP_ENCODELONG(25),  /*produce CSV output*/102, 0},
+	{HLP_ENCODELONG(26),  /*produce DXF output*/156, 0},
+	{HLP_ENCODELONG(27),  /*produce EPS output*/454, 0},
+	{HLP_ENCODELONG(28),  /*produce GPX output*/455, 0},
+	{HLP_ENCODELONG(29),  /*produce HPGL output*/456, 0},
+	{HLP_ENCODELONG(30),  /*produce JSON output*/457, 0},
+	{HLP_ENCODELONG(31),  /*produce KML output*/458, 0},
 	/* TRANSLATORS: "Compass" and "Carto" are the names of software packages,
 	 * so should not be translated. */
-	{HLP_ENCODELONG(31),  /*produce Compass PLT output for Carto*/159, 0},
+	{HLP_ENCODELONG(32),  /*produce Compass PLT output for Carto*/159, 0},
 	/* TRANSLATORS: "Skencil" is the name of a software package, so should not be
 	 * translated. */
-	{HLP_ENCODELONG(32),  /*produce Skencil output*/158, 0},
-	{HLP_ENCODELONG(33),  /*produce Survex POS output*/459, 0},
-	{HLP_ENCODELONG(34),  /*produce SVG output*/160, 0},
+	{HLP_ENCODELONG(33),  /*produce Skencil output*/158, 0},
+	{HLP_ENCODELONG(34),  /*produce Survex POS output*/459, 0},
+	{HLP_ENCODELONG(35),  /*produce SVG output*/160, 0},
 	{0, 0, 0}
    };
 
